@@ -60,6 +60,9 @@ extern "C"
 extern "C" InputPlugin adplug_ip;
 static gboolean audio_error = FALSE;
 GtkWidget *about_win = NULL;
+static GMutex * control_mutex;
+static GCond * control_cond;
+static gboolean pause_flag;
 
 // Configuration (and defaults)
 static struct
@@ -81,13 +84,11 @@ static struct
   char filename[PATH_MAX];
   char *songtitle;
   float time_ms;
-  bool playing;
-  GThread *play_thread;
   GtkLabel *infobox;
   GtkDialog *infodlg;
 } plr =
 {
-0, 0, 0, 0, -1, "", NULL, 0.0f, false, 0, NULL, NULL};
+0, 0, 0, 0, -1, "", NULL, 0.0f, NULL, NULL};
 
 static InputPlayback *playback;
 
@@ -161,7 +162,7 @@ MessageBox (const char *title, const char *text, const char *button)
 
 /***** Dialog boxes *****/
 
-static void
+extern "C" void
 adplug_about (void)
 {
   if (!about_win)
@@ -242,7 +243,7 @@ config_fl_row_unselect (GtkCList * fl, gint row, gint col,
   pl->remove ((CPlayerDesc *) gtk_clist_get_row_data (fl, row));
 }
 
-static void
+extern "C" void
 adplug_config (void)
 {
   GtkDialog *config_dlg = GTK_DIALOG (gtk_dialog_new ());
@@ -476,10 +477,10 @@ factory (VFSFile * fd, Copl * newopl)
   return CAdPlug::factory (fd, newopl, conf.players);
 }
 
-static void adplug_stop (InputPlayback * data);
-static void adplug_play (InputPlayback * data);
-
-
+extern "C" {
+void adplug_stop(InputPlayback * data);
+void adplug_play(InputPlayback * data);
+}
 
 static void
 subsong_slider (GtkAdjustment * adj)
@@ -501,8 +502,8 @@ close_infobox (GtkDialog * infodlg)
   }
 }
 
-static void
-adplug_info_box (char *filename)
+extern "C" void
+adplug_info_box (const gchar *filename)
 {
   CSilentopl tmpopl;
   VFSFile *fd = aud_vfs_buffered_file_new_from_uri (filename);
@@ -674,9 +675,10 @@ adplug_info_box (char *filename)
 
 /***** Main player (!! threaded !!) *****/
 
-static Tuple*
-adplug_get_tuple (char *filename)
+extern "C" Tuple*
+adplug_get_tuple (const gchar *filename)
 {
+  Tuple * ti = NULL;
   CSilentopl tmpopl;
   VFSFile *fd = aud_vfs_buffered_file_new_from_uri (filename);
 
@@ -687,7 +689,8 @@ adplug_get_tuple (char *filename)
 
   if (p)
   {
-    Tuple *ti = aud_tuple_new_from_filename(filename);
+    ti = aud_tuple_new_from_filename (filename);
+
     if (! p->getauthor().empty())
       aud_tuple_associate_string(ti, FIELD_ARTIST, NULL, p->getauthor().c_str());
     if (! p->gettitle().empty())
@@ -700,10 +703,10 @@ adplug_get_tuple (char *filename)
     aud_tuple_associate_string(ti, FIELD_QUALITY, NULL, "sequenced");
     aud_tuple_associate_int(ti, FIELD_LENGTH, NULL, p->songlength (plr.subsong));
     delete p;
-    return ti;
   }
 
-  return NULL;
+  aud_vfs_fclose (fd);
+  return ti;
 }
 
 static char* format_and_free_ti( Tuple* ti, int* length )
@@ -748,11 +751,12 @@ play_loop (void *data)
   CEmuopl opl (conf.freq, conf.bit16, conf.stereo);
   long toadd = 0, i, towrite;
   char *sndbuf, *sndbufpos;
-  int songlength;
+  int songlength = 0;
   bool playing = true,          // Song self-end indicator.
     bit16 = conf.bit16,          // Duplicate config, so it doesn't affect us if
     stereo = conf.stereo;        // the user changes it while we're playing.
   unsigned long freq = conf.freq;
+  gboolean paused = FALSE;
 
   // we use VfsBufferedFile class here because adplug does a lot of
   // probing. a short delay before probing begins is better than
@@ -760,10 +764,7 @@ play_loop (void *data)
   VFSFile *fd = aud_vfs_buffered_file_new_from_uri (playback->filename);
 
   if (!fd)
-  {
-    plr.playing = false;
     return (NULL);
-  }
 
   // Try to load module
   dbg_printf ("factory, ");
@@ -771,7 +772,6 @@ play_loop (void *data)
   {
     dbg_printf ("error!\n");
     // MessageBox("AdPlug :: Error", "File could not be opened!", "Ok");
-    plr.playing = false;
     return (NULL);
   }
 
@@ -798,17 +798,33 @@ play_loop (void *data)
 
   // Set XMMS main window information
   dbg_printf ("xmms, ");
-  playback->set_params (playback, plr.songtitle, plr.songlength, freq * sampsize * 8,
+  playback->set_params (playback, NULL, 0, freq * sampsize * 8,
                       freq, stereo ? 2 : 1);
 
   // Rewind player to right subsong
   dbg_printf ("rewind, ");
   plr.p->rewind (plr.subsong);
+  plr.time_ms = 0;
+
+  g_mutex_lock (control_mutex);
+  plr.seek = -1;
+  pause_flag = FALSE;
+  playback->playing = TRUE;
+  playback->set_pb_ready (playback);
+  g_mutex_unlock (control_mutex);
 
   // main playback loop
   dbg_printf ("loop.\n");
-  while ((playing || conf.endless) && plr.playing)
+  while ((playing || conf.endless))
   {
+    g_mutex_lock (control_mutex);
+
+    if (! playback->playing)
+    {
+        g_mutex_unlock (control_mutex);
+        break;
+    }
+
     // seek requested ?
     if (plr.seek != -1)
     {
@@ -826,8 +842,25 @@ play_loop (void *data)
       // Reset output plugin and some values
       playback->output->flush ((int) plr.time_ms);
       plr.seek = -1;
+      g_cond_signal (control_cond);
     }
 
+    if (pause_flag != paused)
+    {
+        playback->output->pause (pause_flag);
+        paused = pause_flag;
+        g_cond_signal (control_cond);
+    }
+
+    if (paused)
+    {
+        g_cond_wait (control_cond, control_mutex);
+        g_mutex_unlock (control_mutex);
+        continue;
+    }
+
+    g_mutex_unlock (control_mutex);
+
     // fill sound buffer
     towrite = SNDBUFSIZE;
     sndbufpos = sndbuf;
@@ -846,30 +879,21 @@ play_loop (void *data)
       toadd -= (long) (plr.p->getrefresh () * i);
     }
 
-    // write sound buffer
-    playback->pass_audio(playback,
-                   bit16 ? FORMAT_16 : FORMAT_8,
-                   stereo ? 2 : 1, SNDBUFSIZE * sampsize, sndbuf, NULL);
+    playback->output->write_audio (sndbuf, SNDBUFSIZE * sampsize);
 
     // update infobox, if necessary
-    if (plr.infobox && plr.playing)
+    if (plr.infobox)
       update_infobox ();
   }
 
-  // playback finished - deinit
-  dbg_printf ("play_loop(\"%s\"): ", filename);
-  if (!playing)
-  {                             // wait for output plugin to finish if song has self-ended
-    dbg_printf ("wait, ");
-    while (playback->output->buffer_playing ())
+  g_mutex_lock (control_mutex);
+
+  while (playback->playing && playback->output->buffer_playing ())
       g_usleep (10000);
-  }
-  else
-  {                             // or else, flush its output buffers
-    dbg_printf ("flush, ");
-    playback->output->buffer_free ();
-    playback->output->buffer_free ();
-  }
+
+  playback->playing = FALSE;
+  g_cond_signal (control_cond); /* wake up any waiting request */
+  g_mutex_unlock (control_mutex);
 
   // free everything and exit
   dbg_printf ("free");
@@ -881,7 +905,6 @@ play_loop (void *data)
     plr.songtitle = NULL;
   }
   free (sndbuf);
-  plr.playing = false;          // important! XMMS won't get a self-ended song without it.
   dbg_printf (".\n");
   aud_vfs_fclose (fd);
   return (NULL);
@@ -892,8 +915,8 @@ play_loop (void *data)
 
 /***** Informational *****/
 
-static int
-adplug_is_our_fd (gchar * filename, VFSFile * fd)
+extern "C" int
+adplug_is_our_fd (const gchar * filename, VFSFile * fd)
 {
   CSilentopl tmpopl;
 
@@ -912,36 +935,9 @@ adplug_is_our_fd (gchar * filename, VFSFile * fd)
   return FALSE;
 }
 
-static int
-adplug_get_time (InputPlayback * data)
-{
-  if (audio_error)
-  {
-    dbg_printf ("adplug_get_time(): returned -2\n");
-    return -2;
-  }
-  if (!plr.playing)
-  {
-    dbg_printf ("adplug_get_time(): returned -1\n");
-    return -1;
-  }
-  return playback->output->output_time ();
-}
-
-static void
-adplug_song_info (char *filename, char **title, int *length)
-{
-  *length = -1;
-  *title = NULL;
-
-  Tuple* ti = adplug_get_tuple( filename );
-  if ( ti )
-    *title = format_and_free_ti( ti, length );
-}
-
 /***** Player control *****/
 
-static void
+extern "C" void
 adplug_play (InputPlayback * data)
 {
   char *filename = data->filename;
@@ -964,54 +960,59 @@ adplug_play (InputPlayback * data)
     return;
   }
 
-  // Initialize global player data (this is here to prevent a race condition
-  // between adplug_get_time() returning the playback state and adplug_loop()
-  // initializing the playback state)
-  dbg_printf ("init, ");
-  plr.playing = true;
-  plr.time_ms = 0.0f;
-  plr.seek = -1;
-
-  // start player func
-  dbg_printf ("play");
-  plr.play_thread =  g_thread_self();
-  playback->playing = TRUE;
-  playback->set_pb_ready(playback);
-  play_loop(playback);
-  playback->playing = FALSE;
-  dbg_printf (".\n");
-}
-
-static void
-adplug_stop (InputPlayback * playback)
-{
-  dbg_printf ("adplug_stop(): join, ");
-  plr.playing = false;
-  g_thread_join (plr.play_thread);  // stop player thread
-  dbg_printf ("close");
+  play_loop (playback);
   playback->output->close_audio ();
-  dbg_printf (".\n");
 }
 
-static void
-adplug_pause (InputPlayback * playback, short paused)
+extern "C" void adplug_stop (InputPlayback * playback)
 {
-  dbg_printf ("adplug_pause(%d)\n", paused);
-  playback->output->pause (paused);
+    g_mutex_lock (control_mutex);
+
+    if (playback->playing)
+    {
+        playback->playing = FALSE;
+        g_cond_signal (control_cond);
+        g_mutex_unlock (control_mutex);
+        g_thread_join (playback->thread);
+        playback->thread = NULL;
+    }
+    else
+        g_mutex_unlock (control_mutex);
 }
 
-static void
-adplug_seek (InputPlayback * data, int time)
+extern "C" void adplug_pause (InputPlayback * playback, gshort paused)
 {
-  dbg_printf ("adplug_seek(%d)\n", time);
-  plr.seek = time * 1000;       // time is in seconds, but we count in ms
+    g_mutex_lock (control_mutex);
+
+    if (playback->playing)
+    {
+        pause_flag = paused;
+        g_cond_signal (control_cond);
+        g_cond_wait (control_cond, control_mutex);
+    }
+
+    g_mutex_unlock (control_mutex);
+}
+
+extern "C" void adplug_mseek (InputPlayback * data, gulong time)
+{
+    g_mutex_lock (control_mutex);
+
+    if (playback->playing)
+    {
+        plr.seek = time;
+        g_cond_signal (control_cond);
+        g_cond_wait (control_cond, control_mutex);
+    }
+
+    g_mutex_unlock (control_mutex);
 }
 
 /***** Configuration file handling *****/
 
 #define CFG_VERSION "AdPlug"
 
-static void
+extern "C" void
 adplug_init (void)
 {
   dbg_printf ("adplug_init(): open, ");
@@ -1055,15 +1056,20 @@ adplug_init (void)
     {
       std::string userdb;
       userdb = "file://" + std::string(g_get_home_dir()) + "/" ADPLUG_CONFDIR "/" + ADPLUGDB_FILE;
+      if (aud_vfs_file_test(userdb.c_str(),G_FILE_TEST_EXISTS)) {
       plr.db->load (userdb);    // load user's database
       dbg_printf (" (userdb=\"%s\")", userdb.c_str());
+      }
     }
   }
   CAdPlug::set_database (plr.db);
   dbg_printf (".\n");
+
+  control_mutex = g_mutex_new ();
+  control_cond = g_cond_new ();
 }
 
-static void
+extern "C" void
 adplug_quit (void)
 {
   dbg_printf ("adplug_quit(): open, ");
@@ -1099,47 +1105,7 @@ adplug_quit (void)
   dbg_printf ("close");
   aud_cfg_db_close (db);
   dbg_printf (".\n");
+
+  g_mutex_free (control_mutex);
+  g_cond_free (control_cond);
 }
-
-/***** Plugin (exported) *****/
-
-const gchar *fmts[] =
-    { "a2m", "adl", "amd", "bam", "cff", "cmf", "d00", "dfm", "dmo", "dro",
-      "dtm", "hsc", "hsp", "ins", "jbm", "ksm", "laa", "lds", "m", "mad",
-      "mkj", "msc", "rad", "raw", "rix", "rol", "s3m", "sa2", "sat", "sci",
-      "sng", "wlf", "xad", "xsm",
-      NULL };
-
-InputPlugin adplug_ip = {
-  NULL,                         // handle (filled by XMMS)
-  NULL,                         // filename (filled by XMMS)
-  (gchar *)ADPLUG_NAME,                  // plugin description
-  adplug_init,                  // plugin functions...
-  adplug_quit,
-  adplug_about,
-  adplug_config,
-  FALSE,
-  NULL,
-  NULL,
-  adplug_play,
-  adplug_stop,
-  adplug_pause,
-  adplug_seek,
-  adplug_get_time,
-  NULL,                         // get_volume (handled by output plugin)
-  NULL,                         // set_volume (...)
-  NULL,                         // OBSOLETE - DO NOT USE!
-  NULL,                         // add_vis_pcm (filled by XMMS)
-  NULL,                         // set_info (filled by XMMS)
-  NULL,                         // set_info_text (filled by XMMS)
-  adplug_song_info,
-  adplug_info_box,
-  adplug_get_tuple,
-  adplug_is_our_fd,
-  (gchar **)fmts,
-};
-
-InputPlugin *adplug_iplist[] = { &adplug_ip, NULL };
-
-DECLARE_PLUGIN(adplug, NULL, NULL, adplug_iplist, NULL, NULL, NULL, NULL,NULL);
-
