@@ -29,15 +29,9 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <audacious/main.h>
 #include <audacious/plugin.h>
-#include <audacious/playlist.h>
-#include <audacious/configdb.h>
 #include <audacious/i18n.h>
-#include <audacious/util.h>
-#include <libaudcore/vfs.h>
 
-#include <libSAD/libSAD.h>
 
 struct format_info {
     AFormat format;
@@ -59,11 +53,7 @@ typedef struct _FileWriter
     void (*configure)(void);
     gint (*open)(void);
     void (*write)(void *ptr, gint length);
-    void (*flush)(void);
     void (*close)(void);
-    gint (*free)(void);
-    gint (*playing)(void);
-    gint (*get_written_time)(void);
     AFormat format_required;
 } FileWriter;
 
