@@ -23,15 +23,13 @@
 
 #define _ENTER _DEBUG("enter")
 #define _LEAVE _DEBUG("leave"); return
-#define _MESSAGE(tag, string, ...) do { fprintf(stderr, "%s: libflacng.so: %s:%d (%s): " string "\n", \
-    tag, __FILE__, __LINE__, __func__, ##__VA_ARGS__); } while(0)
 
-#define _ERROR(...) _MESSAGE("ERROR", __VA_ARGS__)
+#define _ERROR(...) do { printf ("flacng: " __VA_ARGS__); putchar ('\n'); } while (0)
 
 #ifdef FLACNG_DEBUG
-#define _DEBUG(...) _MESSAGE("DEBUG",  __VA_ARGS__)
+#define _DEBUG _ERROR
 #else
-#define _DEBUG(...) {}
+#define _DEBUG(...)
 #endif
 
 #endif
