@@ -1,14 +1,12 @@
 /*      xmms - jack output plugin
- *	Copyright 2002 Chris Morgan<cmorgan@alum.wpi.edu>
+ *    Copyright 2002 Chris Morgan<cmorgan@alum.wpi.edu>
  *
  *      audacious port (2005-2006) by Giacomo Lozito from develia.org
  *
- *	This code maps xmms calls into the jack translation library
+ *    This code maps xmms calls into the jack translation library
  */
 
 #include <audacious/plugin.h>
-#include <audacious/configdb.h>
-#include <audacious/util.h>
 #include <dlfcn.h>
 #include <gtk/gtk.h>
 #include <audacious/i18n.h>
@@ -30,14 +28,14 @@ jackconfig jack_cfg;
 #define TRACE(...)                                      \
     if(jack_cfg.isTraceEnabled) {                       \
         fprintf(OUTFILE, "%s:", __FUNCTION__),          \
-        fprintf(OUTFILE, __VA_ARGS__),				    \
+        fprintf(OUTFILE, __VA_ARGS__),                    \
         fflush(OUTFILE);                                \
     }
 
 #define ERR(...)                                        \
     if(jack_cfg.isTraceEnabled) {                       \
         fprintf(OUTFILE, "ERR: %s:", __FUNCTION__),     \
-        fprintf(OUTFILE, __VA_ARGS__),				    \
+        fprintf(OUTFILE, __VA_ARGS__),                    \
         fflush(OUTFILE);                                \
     }
 
@@ -55,23 +53,14 @@ static format_info_t input;
 static format_info_t effect;
 static format_info_t output;
 
-static convert_freq_func_t freq_convert; /* rate convert function */
-static struct xmms_convert_buffers *convertb; /* convert buffer */
-
-static int isXmmsFrequencyAvailable = 0;
-
 static gboolean output_opened; /* true if we have a connection to jack */
 
-static GtkWidget *dialog, *button, *label;
-
-
-void jack_set_volume(int l, int r);
 
 /* Giacomo's note: removed the destructor from the original xmms-jack, cause
    destructors + thread join + NPTL currently leads to problems; solved this
    by adding a cleanup function callback for output plugins in Audacious, this
    is used to close the JACK connection and to perform a correct shutdown */
-void jack_cleanup(void)
+static void jack_cleanup(void)
 {
   int errval;
   TRACE("cleanup\n");
@@ -79,48 +68,60 @@ void jack_cleanup(void)
   if((errval = JACK_Close(driver)))
     ERR("error closing device, errval of %d\n", errval);
 
-  /* only clean this up if we have the function to call */
-  if(isXmmsFrequencyAvailable)
-  {
-    aud_convert_buffers_destroy(convertb); /* clean up the rate conversion buffers */
-  }
-
   return;
 }
 
 
-void jack_sample_rate_error(void)
+/* Set the volume */
+static void jack_set_volume(int l, int r)
 {
-	dialog = gtk_dialog_new();
-	gtk_window_set_title(GTK_WINDOW(dialog), _("Sample rate mismatch"));
-	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
-	label = gtk_label_new(_(
-		"Xmms is asking for a sample rate that differs from\n "
-		"that of the jack server.  Xmms 1.2.8 or later\n"
-		"contains resampling routines that xmms-jack will\n"
-		"dynamically load and use to perform resampling.\n"
-		"Or you can restart the jack server\n"
-		"with a sample rate that matches the one that\n"
-		"xmms desires.  -r is the option for the jack\n"
-		"alsa driver so -r 44100 or -r 48000 should do\n\n"
-		"Chris Morgan <cmorgan@alum.wpi.edu>\n"));
+  if(output.channels == 1)
+  {
+    TRACE("l(%d)\n", l);
+  } else if(output.channels > 1)
+  {
+    TRACE("l(%d), r(%d)\n", l, r);
+  }
 
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
-	gtk_widget_show(label);
+  if(output.channels > 0) {
+      JACK_SetVolumeForChannel(driver, 0, l);
+      jack_cfg.volume_left = l;
+  }
+  if(output.channels > 1) {
+      JACK_SetVolumeForChannel(driver, 1, r);
+      jack_cfg.volume_right = r;
+  }
+}
 
-	button = gtk_button_new_with_label(_(" Close "));
-	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, FALSE, FALSE, 0);
-	gtk_widget_show(button);
 
-	gtk_widget_show(dialog);
-	gtk_widget_grab_focus(button);
+/* Get the current volume setting */
+static void jack_get_volume(int *l, int *r)
+{
+  unsigned int _l, _r;
+
+  if(output.channels > 0)
+  {
+      JACK_GetVolumeForChannel(driver, 0, &_l);
+      (*l) = _l;
+  }
+  if(output.channels > 1)
+  {
+      JACK_GetVolumeForChannel(driver, 1, &_r);
+      (*r) = _r;
+  }
+
+#if VERBOSE_OUTPUT
+  if(output.channels == 1)
+      TRACE("l(%d)\n", *l);
+  else if(output.channels > 1)
+      TRACE("l(%d), r(%d)\n", *l, *r);
+#endif
 }
 
 
 /* Return the number of milliseconds of audio data that has been */
 /* written out to the device */
-gint jack_get_written_time(void)
+static gint jack_get_written_time(void)
 {
   long return_val;
   return_val = JACK_GetPosition(driver, MILLISECONDS, WRITTEN);
@@ -132,7 +133,7 @@ gint jack_get_written_time(void)
 
 /* Return the current number of milliseconds of audio data that has */
 /* been played out of the audio device, not including the buffer */
-gint jack_get_output_time(void)
+static gint jack_get_output_time(void)
 {
   gint return_val;
 
@@ -150,7 +151,7 @@ gint jack_get_output_time(void)
 /* returns TRUE if we are currently playing */
 /* NOTE: this was confusing at first BUT, if the device is open and there */
 /* is no more audio to be played, then the device is NOT PLAYING */
-gint jack_playing(void)
+static gint jack_playing(void)
 {
   gint return_val;
 
@@ -191,10 +192,10 @@ void jack_set_port_connection_mode()
 }
 
 /* Initialize necessary things */
-void jack_init(void)
+static OutputPluginInitStatus jack_init(void)
 {
   /* read the isTraceEnabled setting from the config file */
-  ConfigDb *cfgfile;
+  mcs_handle_t *cfgfile;
 
   cfgfile = aud_cfg_db_open();
   if (!cfgfile)
@@ -227,21 +228,15 @@ void jack_init(void)
   /* set the port connection mode */
   jack_set_port_connection_mode();
 
-  isXmmsFrequencyAvailable = 1;
-
-  /* only initialize this stuff if we have the functions available */
-  if(isXmmsFrequencyAvailable)
-  {
-    convertb = aud_convert_buffers_new ();
-    freq_convert = aud_convert_get_frequency_func(FMT_S16_LE, 2);
-  }
-
   output_opened = FALSE;
+  
+  /* Always return OK, as we don't know about physical devices here */
+  return OUTPUT_PLUGIN_INIT_FOUND_DEVICES;
 }
 
 
 /* Return the amount of data that can be written to the device */
-gint jack_free(void)
+static gint audacious_jack_free(void)
 {
   unsigned long return_val = JACK_GetBytesFreeSpace(driver);
   unsigned long tmp;
@@ -270,9 +265,9 @@ gint jack_free(void)
 
 
 /* Close the device */
-void jack_close(void)
+static void jack_close(void)
 {
-  ConfigDb *cfgfile;
+  mcs_handle_t *cfgfile;
 
   cfgfile = aud_cfg_db_open();
   aud_cfg_db_set_int(cfgfile, "jack", "volume_left", jack_cfg.volume_left); /* stores the volume setting */
@@ -287,14 +282,14 @@ void jack_close(void)
 
 
 /* Open the device up */
-gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
+static gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
 {
   int bits_per_sample;
   int retval;
   unsigned long rate;
 
   TRACE("fmt == %d, sample_rate == %d, num_channels == %d\n",
-	fmt, sample_rate, num_channels);
+    fmt, sample_rate, num_channels);
 
   if((fmt == FMT_U8) || (fmt == FMT_S8))
   {
@@ -345,26 +340,9 @@ gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
   retval = JACK_Open(&driver, bits_per_sample, &rate, output.channels);
   output.frequency = rate; /* avoid compile warning as output.frequency differs in type
                               from what JACK_Open() wants for the type of the rate parameter */
-  if((retval == ERR_RATE_MISMATCH) && isXmmsFrequencyAvailable)
+  if((retval == ERR_RATE_MISMATCH))
   {
-    TRACE("xmms(input) wants rate of '%ld', jacks rate(output) is '%ld', opening at jack rate\n", input.frequency, output.frequency);
-
-    /* open the jack device with true jack's rate, return 0 upon failure */
-    retval = JACK_Open(&driver, bits_per_sample, &rate, output.channels);
-    output.frequency = rate; /* avoid compile warning as output.frequency differs in type
-                                from what JACK_Open() wants for the type of the rate parameter */
-    if(retval)
-    {
-      TRACE("failed to open jack with JACK_Open(), error %d\n", retval);
-      return 0;
-    }
-    TRACE("success!!\n");
-  } else if((retval == ERR_RATE_MISMATCH) && !isXmmsFrequencyAvailable)
-  {
-    TRACE("JACK_Open(), sample rate mismatch with no resampling routines available\n");
-
-    jack_sample_rate_error(); /* notify the user that we can't resample */
-
+    TRACE("set the resampling rate properly");
     return 0;
   } else if(retval != ERR_SUCCESS)
   {
@@ -380,7 +358,7 @@ gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
 
 
 /* write some audio out to the device */
-void jack_write(gpointer ptr, gint length)
+static void jack_write(gpointer ptr, gint length)
 {
   long written;
 
@@ -405,7 +383,7 @@ void jack_write(gpointer ptr, gint length)
 /* the number of milliseconds of offset passed in */
 /* This is done so the driver itself keeps track of */
 /* current playing position of the mp3 */
-void jack_flush(gint ms_offset_time)
+static void jack_flush(gint ms_offset_time)
 {
   TRACE("setting values for ms_offset_time of %d\n", ms_offset_time);
 
@@ -419,7 +397,7 @@ void jack_flush(gint ms_offset_time)
 
 
 /* Pause the jack device */
-void jack_pause(short p)
+static void jack_pause(short p)
 {
   TRACE("p == %d\n", p);
 
@@ -432,97 +410,49 @@ void jack_pause(short p)
 }
 
 
-/* Set the volume */
-void jack_set_volume(int l, int r)
+static void jack_about(void)
 {
-  if(output.channels == 1)
-  {
-    TRACE("l(%d)\n", l);
-  } else if(output.channels > 1)
-  {
-    TRACE("l(%d), r(%d)\n", l, r);
-  }
+    static GtkWidget *aboutbox = NULL;
 
-  if(output.channels > 0) {
-      JACK_SetVolumeForChannel(driver, 0, l);
-      jack_cfg.volume_left = l;
-  }
-  if(output.channels > 1) {
-      JACK_SetVolumeForChannel(driver, 1, r);
-      jack_cfg.volume_right = r;
-  }
+    if (aboutbox == NULL)
+    {
+        aboutbox = audacious_info_dialog(
+            _("About JACK Output Plugin 0.17"),
+            _("XMMS jack Driver 0.17\n\n"
+              "xmms-jack.sf.net\nChris Morgan<cmorgan@alum.wpi.edu>\n\n"
+              "Audacious port by\nGiacomo Lozito from develia.org"),
+            _("Ok"), FALSE, NULL, NULL);
+        g_signal_connect(GTK_OBJECT(aboutbox), "destroy",
+                   (GCallback)gtk_widget_destroyed, &aboutbox);
+    }
 }
 
-
-/* Get the current volume setting */
-void jack_get_volume(int *l, int *r)
+static void jack_tell_audio(AFormat * fmt, gint * srate, gint * nch)
 {
-  unsigned int _l, _r;
-
-  if(output.channels > 0)
-  {
-      JACK_GetVolumeForChannel(driver, 0, &_l);
-      (*l) = _l;
-  }
-  if(output.channels > 1)
-  {
-      JACK_GetVolumeForChannel(driver, 1, &_r);
-      (*r) = _r;
-  }
-
-#if VERBOSE_OUTPUT
-  if(output.channels == 1)
-      TRACE("l(%d)\n", *l);
-  else if(output.channels > 1)
-      TRACE("l(%d), r(%d)\n", *l, *r);
-#endif
-}
-
-
-void jack_about(void)
-{
-	static GtkWidget *aboutbox = NULL;
-
-	if ( aboutbox == NULL )
-	{
-		aboutbox = audacious_info_dialog(
-			_("About JACK Output Plugin 0.17"),
-			_("XMMS jack Driver 0.17\n\n"
-			  "xmms-jack.sf.net\nChris Morgan<cmorgan@alum.wpi.edu>\n\n"
-			  "Audacious port by\nGiacomo Lozito from develia.org"),
-			_("Ok"), FALSE, NULL, NULL);
-		g_signal_connect(GTK_OBJECT(aboutbox), "destroy",
-				   (GCallback)gtk_widget_destroyed, &aboutbox);
-	}
-}
-
-static void
-jack_tell_audio(AFormat * fmt, gint * srate, gint * nch)
-{
-	(*fmt) = input.format;
-	(*srate) = input.frequency;
-	(*nch) = input.channels;
+    (*fmt) = input.format;
+    (*srate) = input.frequency;
+    (*nch) = input.channels;
 }
 
 OutputPlugin jack_op =
 {
-	.description = "JACK Output Plugin 0.17",
-	.init = jack_init,
-	.cleanup = jack_cleanup,
-	.about = jack_about,
-	.configure = jack_configure,
-	.get_volume = jack_get_volume,
-	.set_volume = jack_set_volume,
-	.open_audio = jack_open,
-	.write_audio = jack_write,
-	.close_audio = jack_close,
-	.flush = jack_flush,
-	.pause = jack_pause,
-	.buffer_free = jack_free,
-	.buffer_playing = jack_playing,
-	.output_time = jack_get_output_time,
-	.written_time = jack_get_written_time,
-	.tell_audio = jack_tell_audio
+    .description = "JACK Output Plugin 0.17",
+    .init = jack_init,
+    .cleanup = jack_cleanup,
+    .about = jack_about,
+    .configure = jack_configure,
+    .get_volume = jack_get_volume,
+    .set_volume = jack_set_volume,
+    .open_audio = jack_open,
+    .write_audio = jack_write,
+    .close_audio = jack_close,
+    .flush = jack_flush,
+    .pause = jack_pause,
+    .buffer_free = audacious_jack_free,
+    .buffer_playing = jack_playing,
+    .output_time = jack_get_output_time,
+    .written_time = jack_get_written_time,
+    .tell_audio = jack_tell_audio
 };
 
 OutputPlugin *jack_oplist[] = { &jack_op, NULL };
