@@ -32,6 +32,8 @@
 
 #include <pulse/pulseaudio.h>
 
+#define ERROR(...) do {fprintf (stderr, "pulseaudio: " __VA_ARGS__); putchar ('\n');} while (0)
+
 static pa_context *context = NULL;
 static pa_stream *stream = NULL;
 static pa_threaded_mainloop *mainloop = NULL;
@@ -40,11 +42,11 @@ static pa_cvolume volume;
 static int volume_valid = 0;
 
 static int do_trigger = 0;
-static uint64_t written = 0;
-static int time_offset_msec = 0;
-static int just_flushed = 0;
+static int64_t written;
+static int bytes_per_second;
 
 static int connected = 0;
+static int cached_time = 0;
 
 static pa_time_event *volume_time_event = NULL;
 
@@ -65,14 +67,10 @@ do { \
 
 static const char* get_song_name(void) {
     static char t[256];
-    gint pos;
     char *str, *u;
-    Playlist *playlist = aud_playlist_get_active();
 
-    pos = aud_playlist_get_position(playlist);
-    if (!(str = aud_playlist_get_songtitle(playlist, pos)))
+    if (!(str = aud_playback_get_title()))
         return "Playback Stream";
-
     snprintf(t, sizeof(t), "%s", u = pa_locale_to_utf8(str));
     pa_xfree(u);
 
@@ -223,6 +221,10 @@ static void pulse_set_volume(int l, int r) {
         CHECK_DEAD_GOTO(fail, 1);
     }
 
+    /* sanitize output volumes. */
+    l = CLAMP(l, 0, 100);
+    r = CLAMP(r, 0, 100);
+
     if (!volume_valid || volume.channels !=  1) {
         volume.values[0] = ((pa_volume_t) l * PA_VOLUME_NORM)/100;
         volume.values[1] = ((pa_volume_t) r * PA_VOLUME_NORM)/100;
@@ -327,7 +329,7 @@ static int pulse_get_written_time(void) {
     pa_threaded_mainloop_lock(mainloop);
     CHECK_DEAD_GOTO(fail, 1);
 
-    r = (int) (((double) written*1000) / pa_bytes_per_second(pa_stream_get_sample_spec(stream)));
+    r = written * (int64_t) 1000 / bytes_per_second;
 
 fail:
     pa_threaded_mainloop_unlock(mainloop);
@@ -335,71 +337,81 @@ fail:
     return r;
 }
 
-static int pulse_get_output_time(void) {
-    int r = 0;
-    pa_usec_t t;
+static void pulse_set_written_time (int time)
+{
+    pa_threaded_mainloop_lock (mainloop);
 
-    CHECK_CONNECTED(0);
+    written = time * (int64_t) bytes_per_second / 1000;
+    pa_stream_set_name (stream, get_song_name (), stream_success_cb, NULL);
 
-    pa_threaded_mainloop_lock(mainloop);
-
-    for (;;) {
-        CHECK_DEAD_GOTO(fail, 1);
-
-        if (pa_stream_get_time(stream, &t) >= 0)
-            break;
-
-        if (pa_context_errno(context) != PA_ERR_NODATA) {
-            AUDDBG("pa_stream_get_time() failed: %s", pa_strerror(pa_context_errno(context)));
-            goto fail;
-        }
-
-        pa_threaded_mainloop_wait(mainloop);
-    }
-
-    r = (int) (t / 1000);
-
-    if (just_flushed) {
-        time_offset_msec -= r;
-        just_flushed = 0;
-    }
-
-    r += time_offset_msec;
-
-fail:
-    pa_threaded_mainloop_unlock(mainloop);
-
-    return r;
+    pa_threaded_mainloop_unlock (mainloop);
 }
 
-static int pulse_playing(void) {
-    int r = 0;
-    const pa_timing_info *i;
+static int pulse_get_output_time (void)
+{
+    int time = 0;
+    const pa_timing_info * timing;
+    struct timeval now;
 
     CHECK_CONNECTED(0);
 
     pa_threaded_mainloop_lock(mainloop);
 
-    for (;;) {
-        CHECK_DEAD_GOTO(fail, 1);
+    if ((timing = pa_stream_get_timing_info (stream)) == NULL)
+        goto fail;
 
-        if ((i = pa_stream_get_timing_info(stream)))
-            break;
+    gettimeofday (& now, NULL);
 
-        if (pa_context_errno(context) != PA_ERR_NODATA) {
-            AUDDBG("pa_stream_get_timing_info() failed: %s", pa_strerror(pa_context_errno(context)));
-            goto fail;
-        }
+    time = (int) ((written - (timing->write_index - timing->read_index)) *
+     (int64_t) 1000 / bytes_per_second) - (int) (timing->transport_usec / 1000)
+     - (int) (timing->sink_usec / 1000) + (int) ((now.tv_sec -
+     timing->timestamp.tv_sec) * 1000) + (int) ((now.tv_usec -
+     timing->timestamp.tv_usec) / 1000);
 
-        pa_threaded_mainloop_wait(mainloop);
+    if (pa_stream_is_corked(stream))
+    {
+	int delta = time - cached_time;
+	if (delta > 0 && delta < 2000)
+             time = cached_time;
+        else
+             cached_time = time;
+    } else {
+        cached_time = time;
     }
 
-    r = i->playing;
-
 fail:
     pa_threaded_mainloop_unlock(mainloop);
 
-    return r;
+    return time;
+}
+
+static void pulse_drain(void) {
+    pa_operation *o = NULL;
+    int success = 0;
+
+    CHECK_CONNECTED();
+
+    pa_threaded_mainloop_lock(mainloop);
+    CHECK_DEAD_GOTO(fail, 0);
+
+    if (!(o = pa_stream_drain(stream, stream_success_cb, &success))) {
+        AUDDBG("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
+        goto fail;
+    }
+
+    while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
+        CHECK_DEAD_GOTO(fail, 1);
+        pa_threaded_mainloop_wait(mainloop);
+    }
+
+    if (!success)
+        AUDDBG("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
+
+fail:
+    if (o)
+        pa_operation_unref(o);
+
+    pa_threaded_mainloop_unlock(mainloop);
 }
 
 static void pulse_flush(int time) {
@@ -411,10 +423,7 @@ static void pulse_flush(int time) {
     pa_threaded_mainloop_lock(mainloop);
     CHECK_DEAD_GOTO(fail, 1);
 
-    /* gapless playback: new stream, reset the title. --nenolod */
-    if (time == 0) {
-        pa_stream_set_name(stream, get_song_name(), stream_success_cb, &success);
-    }
+    written = time * (int64_t) bytes_per_second / 1000;
 
     if (!(o = pa_stream_flush(stream, stream_success_cb, &success))) {
         AUDDBG("pa_stream_flush() failed: %s", pa_strerror(pa_context_errno(context)));
@@ -429,10 +438,6 @@ static void pulse_flush(int time) {
     if (!success)
         AUDDBG("pa_stream_flush() failed: %s", pa_strerror(pa_context_errno(context)));
 
-    written = (uint64_t) (((double) time * pa_bytes_per_second(pa_stream_get_sample_spec(stream))) / 1000);
-    just_flushed = 1;
-    time_offset_msec = time;
-
 fail:
     if (o)
         pa_operation_unref(o);
@@ -475,39 +480,8 @@ fail:
     pa_threaded_mainloop_unlock(mainloop);
 }
 
-static void drain(void) {
-    pa_operation *o = NULL;
-    int success = 0;
-
-    CHECK_CONNECTED();
-
-    pa_threaded_mainloop_lock(mainloop);
-    CHECK_DEAD_GOTO(fail, 0);
-
-    if (!(o = pa_stream_drain(stream, stream_success_cb, &success))) {
-        AUDDBG("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
-        goto fail;
-    }
-
-    while (pa_operation_get_state(o) != PA_OPERATION_DONE) {
-        CHECK_DEAD_GOTO(fail, 1);
-        pa_threaded_mainloop_wait(mainloop);
-    }
-
-    if (!success)
-        AUDDBG("pa_stream_drain() failed: %s", pa_strerror(pa_context_errno(context)));
-
-fail:
-    if (o)
-        pa_operation_unref(o);
-
-    pa_threaded_mainloop_unlock(mainloop);
-}
-
 static void pulse_close(void)
 {
-    drain();
-
     connected = 0;
 
     if (mainloop)
@@ -533,30 +507,6 @@ static void pulse_close(void)
     volume_time_event = NULL;
 }
 
-static OutputPluginInitStatus pulse_init(void) {
-    pa_sample_spec ss;
-
-    g_assert(!mainloop);
-    g_assert(!context);
-    g_assert(!stream);
-    g_assert(!connected);
-
-    ss.format = PA_SAMPLE_S16NE;
-    ss.rate = 44100;
-    ss.channels = 2;
-
-    if (!pa_sample_spec_valid(&ss))
-        return OUTPUT_PLUGIN_INIT_FAIL;
-
-    if (!(mainloop = pa_threaded_mainloop_new())) {
-        pulse_close();
-        return OUTPUT_PLUGIN_INIT_FAIL;
-    }
-
-    pulse_close();
-    return OUTPUT_PLUGIN_INIT_FOUND_DEVICES;
-}
-
 static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_sample_spec ss;
     pa_operation *o = NULL;
@@ -578,29 +528,22 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
         case FMT_S16_BE:
             ss.format = PA_SAMPLE_S16BE;
             break;
-        case FMT_S16_NE:
-            ss.format = PA_SAMPLE_S16NE;
-            break;
-#if 0 /* pulseaudio only handles 16 and 32bit formats*/
+
+#ifdef PA_SAMPLE_S24_32LE
         case FMT_S24_LE:
-            ss.format = PA_SAMPLE_S24LE;
+            ss.format = PA_SAMPLE_S24_32LE;
             break;
         case FMT_S24_BE:
-            ss.format = PA_SAMPLE_S24BE;
-            break;
-        case FMT_S24_NE:
-            ss.format = PA_SAMPLE_S24NE;
+            ss.format = PA_SAMPLE_S24_32BE;
             break;
 #endif
+
         case FMT_S32_LE:
             ss.format = PA_SAMPLE_S32LE;
             break;
         case FMT_S32_BE:
             ss.format = PA_SAMPLE_S32BE;
             break;
-        case FMT_S32_NE:
-            ss.format = PA_SAMPLE_S32NE;
-            break;
 	case FMT_FLOAT:
             ss.format = PA_SAMPLE_FLOAT32NE;
             break;
@@ -613,21 +556,15 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     if (!pa_sample_spec_valid(&ss))
         return FALSE;
 
-    if (!volume_valid) {
-        pa_cvolume_reset(&volume, ss.channels);
-        volume_valid = 1;
-    } else if (volume.channels != ss.channels)
-        pa_cvolume_set(&volume, ss.channels, pa_cvolume_avg(&volume));
-
     if (!(mainloop = pa_threaded_mainloop_new())) {
-        AUDDBG("Failed to allocate main loop");
+        ERROR ("Failed to allocate main loop");
         goto fail;
     }
 
     pa_threaded_mainloop_lock(mainloop);
 
     if (!(context = pa_context_new(pa_threaded_mainloop_get_api(mainloop), "Audacious"))) {
-        AUDDBG("Failed to allocate context");
+        ERROR ("Failed to allocate context");
         goto unlock_and_fail;
     }
 
@@ -635,12 +572,12 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_context_set_subscribe_callback(context, subscribe_cb, NULL);
 
     if (pa_context_connect(context, NULL, 0, NULL) < 0) {
-        AUDDBG("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
     if (pa_threaded_mainloop_start(mainloop) < 0) {
-        AUDDBG("Failed to start main loop");
+        ERROR ("Failed to start main loop");
         goto unlock_and_fail;
     }
 
@@ -648,12 +585,12 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_threaded_mainloop_wait(mainloop);
 
     if (pa_context_get_state(context) != PA_CONTEXT_READY) {
-        AUDDBG("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to connect to server: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
     if (!(stream = pa_stream_new(context, get_song_name(), &ss, NULL))) {
-        AUDDBG("Failed to create stream: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to create stream: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -661,8 +598,9 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_stream_set_write_callback(stream, stream_request_cb, NULL);
     pa_stream_set_latency_update_callback(stream, stream_latency_update_cb, NULL);
 
-    if (pa_stream_connect_playback(stream, NULL, NULL, PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE, &volume, NULL) < 0) {
-        AUDDBG("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
+    /* Connect stream with sink and default volume */
+    if (pa_stream_connect_playback(stream, NULL, NULL, PA_STREAM_INTERPOLATE_TIMING|PA_STREAM_AUTO_TIMING_UPDATE, NULL, NULL) < 0) {
+        ERROR ("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -670,13 +608,13 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     pa_threaded_mainloop_wait(mainloop);
 
     if (pa_stream_get_state(stream) != PA_STREAM_READY) {
-        AUDDBG("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("Failed to connect stream: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
     /* Now subscribe to events */
     if (!(o = pa_context_subscribe(context, PA_SUBSCRIPTION_MASK_SINK_INPUT, context_success_cb, &success))) {
-        AUDDBG("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -687,7 +625,7 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     }
 
     if (!success) {
-        AUDDBG("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_subscribe() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -695,7 +633,7 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
 
     /* Now request the initial stream info */
     if (!(o = pa_context_get_sink_input_info(context, pa_stream_get_index(stream), info_cb, NULL))) {
-        AUDDBG("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
 
@@ -705,14 +643,14 @@ static int pulse_open(AFormat fmt, int rate, int nch) {
     }
 
     if (!volume_valid) {
-        AUDDBG("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
+        ERROR ("pa_context_get_sink_input_info() failed: %s", pa_strerror(pa_context_errno(context)));
         goto unlock_and_fail;
     }
+    pa_operation_unref(o);
 
     do_trigger = 0;
     written = 0;
-    time_offset_msec = 0;
-    just_flushed = 0;
+    bytes_per_second = FMT_SIZEOF (fmt) * nch * rate;
     connected = 1;
     volume_time_event = NULL;
 
@@ -734,6 +672,15 @@ fail:
     return FALSE;
 }
 
+static OutputPluginInitStatus pulse_init (void)
+{
+    if (! pulse_open (FMT_S16_NE, 44100, 2))
+        return OUTPUT_PLUGIN_INIT_FAIL;
+
+    pulse_close ();
+    return OUTPUT_PLUGIN_INIT_FOUND_DEVICES;
+}
+
 static void pulse_about(void) {
     static GtkWidget *dialog;
 
@@ -782,9 +729,10 @@ static OutputPlugin pulse_op = {
         .flush = pulse_flush,
         .pause = pulse_pause,
         .buffer_free = pulse_free,
-        .buffer_playing = pulse_playing,
+        .drain = pulse_drain,
         .output_time = pulse_get_output_time,
         .written_time = pulse_get_written_time,
+        .set_written_time = pulse_set_written_time,
 };
 
 OutputPlugin *pulse_oplist[] = { &pulse_op, NULL };
