@@ -185,8 +185,6 @@ aud_active_skin_load(const gchar * path)
     ui_skinned_window_draw_all(equalizerwin);
     ui_skinned_window_draw_all(playlistwin);
 
-    playlistwin_update ();
-
     SkinPixmap *pixmap;
     pixmap = &aud_active_skin->pixmaps[SKIN_POSBAR];
     /* last 59 pixels of SKIN_POSBAR are knobs (normal and selected) */
@@ -195,14 +193,13 @@ aud_active_skin_load(const gchar * path)
     return TRUE;
 }
 
-void
-skin_pixmap_free(SkinPixmap * p)
+static void skin_pixmap_free (SkinPixmap * pixmap)
 {
-    g_return_if_fail(p != NULL);
-    g_return_if_fail(p->pixbuf != NULL);
-
-    g_object_unref(p->pixbuf);
-    p->pixbuf = NULL;
+    if (pixmap->pixbuf != NULL)
+    {
+        g_object_unref (pixmap->pixbuf);
+        pixmap->pixbuf = NULL;
+    }
 }
 
 Skin *
@@ -295,31 +292,18 @@ skin_set_default_vis_color(Skin * skin)
            sizeof(skin_default_viscolor));
 }
 
-gchar *
-skin_pixmap_locate(const gchar * dirname, gchar ** basenames)
+gchar * skin_pixmap_locate (const gchar * dirname, gchar * * basenames)
 {
-    gchar *filename;
+    gchar * filename = NULL;
     gint i;
 
-    for (i = 0; basenames[i]; i++)
+    for (i = 0; basenames[i] != NULL; i ++)
     {
-        filename = g_strdup_printf ("%s/%s", dirname, basenames[i]);
-
-        if (aud_vfs_file_test (filename, G_FILE_TEST_IS_REGULAR))
-            return filename;
-
-        g_free (filename);
+        if ((filename = find_file_case_path (dirname, basenames[i])) != NULL)
+            break;
     }
 
-    // Case-insensitive search; much slower.
-    for (i = 0; basenames[i]; i++)
-    if (!(filename = find_path_recursively(dirname, basenames[i])))
-        g_free(filename);
-    else
-        return filename;
-
-    /* can't find any targets -- sorry */
-    return NULL;
+    return filename;
 }
 
 /**
@@ -407,8 +391,20 @@ skin_load_pixmap_id(Skin * skin, SkinPixmapId id, const gchar * path_p)
 
     pm = &skin->pixmaps[id];
     GdkPixbuf *pix = gdk_pixbuf_new_from_file(filename, NULL);
-    pm->pixbuf = audacious_create_colorized_pixbuf(pix, config.colorize_r, config.colorize_g, config.colorize_b);
-    g_object_unref(pix);
+
+    if (pix == NULL)
+        return FALSE;
+
+    if (config.colorize_r == 255 && config.colorize_g == 255 &&
+     config.colorize_b == 255)
+        pm->pixbuf = pix;
+    else
+    {
+        pm->pixbuf = audacious_create_colorized_pixbuf(pix, config.colorize_r,
+         config.colorize_g, config.colorize_b);
+        g_object_unref(pix);
+    }
+
     pm->width = gdk_pixbuf_get_width(pm->pixbuf);
     pm->height = gdk_pixbuf_get_height(pm->pixbuf);
     pm->current_width = pm->width;
@@ -648,7 +644,7 @@ skin_parse_hints(Skin * skin, gchar *path_p)
     if (path_p == NULL)
         return;
 
-    filename = find_file_recursively(path_p, "skin.hints");
+    filename = find_file_case_uri (path_p, "skin.hints");
 
     if (filename == NULL)
         return;
@@ -1265,8 +1261,8 @@ skin_create_transparent_mask(const gchar * path,
     guint i, j;
     gint k;
 
-    if (path)
-        filename = find_file_recursively(path, file);
+    if (path != NULL)
+        filename = find_file_case_uri (path, file);
 
     /* filename will be null if path wasn't set */
     if (!filename)
@@ -1328,45 +1324,41 @@ skin_create_transparent_mask(const gchar * path,
     return mask;
 }
 
-void
-skin_load_viscolor(Skin * skin, const gchar * path, const gchar * basename)
+static void skin_load_viscolor (Skin * skin, const gchar * path, const gchar *
+ basename)
 {
-    VFSFile *file;
-    gint i, c;
-    gchar line[256], *filename;
-    GArray *a;
+    gchar * filename, * buffer, * string, * next;
+    gint line;
 
-    g_return_if_fail(skin != NULL);
-    g_return_if_fail(path != NULL);
-    g_return_if_fail(basename != NULL);
+    skin_set_default_vis_color (skin);
 
-    skin_set_default_vis_color(skin);
-
-    filename = find_file_recursively(path, basename);
-    if (!filename)
+    filename = find_file_case_uri (path, basename);
+    if (filename == NULL)
         return;
 
-    if (!(file = aud_vfs_fopen(filename, "r"))) {
-        g_free(filename);
-        return;
-    }
+    buffer = load_text_file (filename);
+    string = buffer;
 
-    g_free(filename);
+    for (line = 0; string != NULL && line < 24; line ++)
+    {
+        GArray * array;
+        gint column;
 
-    for (i = 0; i < 24; i++) {
-        if (aud_vfs_fgets(line, 255, file)) {
-            a = string_to_garray(line);
-            if (a->len > 2) {
-                for (c = 0; c < 3; c++)
-                    skin->vis_color[i][c] = g_array_index(a, gint, c);
-            }
-            g_array_free(a, TRUE);
+        next = text_parse_line (string);
+        array = string_to_garray (string);
+
+        if (array->len >= 3)
+        {
+            for (column = 0; column < 3; column ++)
+                skin->vis_color[line][column] = g_array_index (array, gint,
+                 column);
         }
-        else
-            break;
+
+        g_array_free (array, TRUE);
+        string = next;
     }
 
-    aud_vfs_fclose(file);
+    g_free (buffer);
 }
 
 static void
@@ -1421,8 +1413,8 @@ skin_load_pixmaps(Skin * skin, const gchar * path)
         skin->pixmaps[SKIN_NUMBERS].width < 108 )
         skin_numbers_generate_dash(skin);
 
-    filename = find_file_recursively(path, "pledit.txt");
-    inifile = aud_open_ini_file(filename);
+    filename = find_file_case_uri (path, "pledit.txt");
+    inifile = (filename != NULL) ? aud_open_ini_file (filename) : NULL;
 
     skin->colors[SKIN_PLEDIT_NORMAL] =
         skin_load_color(inifile, "Text", "Normal", "#2499ff");
@@ -1563,11 +1555,14 @@ skin_load_nolock(Skin * skin, const gchar * path, gboolean force)
     }
 
 #ifndef _WIN32
-    if (!config.disable_inline_gtk && !archive) {
-        gtkrcpath = find_path_recursively(skin->path, "gtkrc");
-        if (gtkrcpath != NULL)
-            skin_set_gtk_theme(settings, skin);
-        g_free(gtkrcpath);
+    if (! config.disable_inline_gtk && ! archive)
+    {
+        gtkrcpath = g_strdup_printf ("%s/gtk-2.0/gtkrc", skin->path);
+
+        if (g_file_test (gtkrcpath, G_FILE_TEST_IS_REGULAR))
+            skin_set_gtk_theme (settings, skin);
+
+        g_free (gtkrcpath);
     }
 #endif
 
