@@ -2,6 +2,7 @@
  * Audacious - a cross-platform multimedia player
  * Copyright (c) 2007 Tomasz Moń
  * Copyright (c) 2008 William Pitcock
+ * Copyright 2009 John Lindgren
  *
  * Based on:
  * BMP - Cross-platform multimedia player
@@ -61,13 +62,10 @@ enum {DRAG_SELECT = 1, DRAG_MOVE};
 struct _UiSkinnedPlaylistPrivate {
     GtkWidget * slider;
     PangoFontDescription * font;
-    SkinPixmapId     skin_index;
-    int width, height, ascent, descent, letter_width, digit_width;
-    char slanted;
-    gint             resize_width, resize_height;
-    int row_height, offset, rows, first, focused;
-    char drag;
-    int scroll, scroll_source, hover;
+    gint width, height, resize_width, resize_height, ascent, descent,
+     letter_width, digit_width, row_height, offset, rows, first, focused,
+     scroll, scroll_source, hover;
+    gboolean slanted, drag;
 };
 
 static void ui_skinned_playlist_class_init         (UiSkinnedPlaylistClass *klass);
@@ -139,7 +137,7 @@ static void ui_skinned_playlist_init(UiSkinnedPlaylist *playlist) {
     priv->rows = 0;
     priv->first = 0;
     priv->focused = -1;
-    priv->drag = 0;
+    priv->drag = FALSE;
     priv->scroll = 0;
     priv->hover = -1;
 
@@ -152,16 +150,15 @@ static void ui_skinned_playlist_init(UiSkinnedPlaylist *playlist) {
     g_object_set_data(G_OBJECT(playlist), "popup_position", GINT_TO_POINTER(-1));
 }
 
-GtkWidget * ui_skinned_playlist_new (GtkWidget * fixed, int x, int y, int width,
- int height, char * font)
+GtkWidget * ui_skinned_playlist_new (GtkWidget * fixed, gint x, gint y, gint
+ width, gint height, const gchar * font)
 {
     UiSkinnedPlaylist *hs = g_object_new (ui_skinned_playlist_get_type (), NULL);
     UiSkinnedPlaylistPrivate *priv = UI_SKINNED_PLAYLIST_GET_PRIVATE(hs);
 
     priv->width = width;
     priv->height = height;
-    priv->slider = 0;
-    priv->skin_index = SKIN_PLEDIT;
+    priv->slider = NULL;
 
     ui_skinned_playlist_set_font ((GtkWidget *) hs, font);
 
@@ -179,7 +176,7 @@ void ui_skinned_playlist_set_slider (GtkWidget * list, GtkWidget * slider)
 
 static void cancel_all (GtkWidget * widget, UiSkinnedPlaylistPrivate * private)
 {
-    private->drag = 0;
+    private->drag = FALSE;
 
     if (private->scroll)
     {
@@ -187,7 +184,7 @@ static void cancel_all (GtkWidget * widget, UiSkinnedPlaylistPrivate * private)
         g_source_remove (private->scroll_source);
     }
 
-    if (private->hover)
+    if (private->hover != -1)
     {
         private->hover = -1;
         gtk_widget_queue_draw (widget);
@@ -250,7 +247,7 @@ static void calc_layout (UiSkinnedPlaylistPrivate * private)
     private->row_height = private->ascent - private->descent;
     private->rows = private->height / private->row_height;
 
-    if (private->rows && active_title)
+    if (private->rows && active_title != NULL)
     {
         private->offset = private->row_height;
         private->rows --;
@@ -266,20 +263,10 @@ static void calc_layout (UiSkinnedPlaylistPrivate * private)
 
 static void scroll_to (UiSkinnedPlaylistPrivate * private, int position)
 {
-    if (position < private->first)
-    {
+    if (position < private->first || position >= private->first + private->rows)
         private->first = position - private->rows / 2;
 
-        if (private->first < 0)
-            private->first = 0;
-    }
-    else if (position > private->first + private->rows - 1)
-    {
-        private->first = position - private->rows / 2;
-
-        if (private->first + private->rows > active_length)
-            private->first = active_length - private->rows;
-    }
+    calc_layout (private);
 }
 
 static void ui_skinned_playlist_size_allocate(GtkWidget *widget, GtkAllocation *allocation) {
@@ -300,7 +287,7 @@ static void ui_skinned_playlist_size_allocate(GtkWidget *widget, GtkAllocation *
     calc_layout (priv);
     gtk_widget_queue_draw (widget);
 
-    if (priv->slider)
+    if (priv->slider != NULL)
         ui_skinned_playlist_slider_update (priv->slider);
 }
 
@@ -313,13 +300,11 @@ playlist_list_draw_string(cairo_t *cr, UiSkinnedPlaylist *pl,
                           guint ppos)
 {
     UiSkinnedPlaylistPrivate * private;
-    int plist_length_int, padding;
+    gint plist_length_int, padding;
     PangoLayout *layout;
 
     private = UI_SKINNED_PLAYLIST_GET_PRIVATE (pl);
 
-    REQUIRE_LOCK (active_playlist->mutex);
-
     cairo_new_path(cr);
 
     if (aud_cfg->show_numbers_in_pl)
@@ -369,12 +354,8 @@ playlist_list_draw_string(cairo_t *cr, UiSkinnedPlaylist *pl,
 static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *event) {
     UiSkinnedPlaylist *pl = UI_SKINNED_PLAYLIST (widget);
     UiSkinnedPlaylistPrivate *priv = UI_SKINNED_PLAYLIST_GET_PRIVATE(pl);
-    g_return_val_if_fail (priv->width > 0 && priv->height > 0, FALSE);
-
-    PlaylistEntry *entry;
-    GList *list;
+    gint active_entry = aud_playlist_get_position (active_playlist);
     PangoLayout *layout;
-    gchar *title;
     gint width, height;
     gint i;
     guint padding, padding_dwidth;
@@ -396,7 +377,6 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
     gint plw_w, plw_h;
 
     cairo_t *cr;
-    gint pos;
 
     g_return_val_if_fail (widget != NULL, FALSE);
     g_return_val_if_fail (UI_SKINNED_IS_PLAYLIST (widget), FALSE);
@@ -420,46 +400,28 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
         layout = gtk_widget_create_pango_layout (widget, active_title);
         pango_layout_set_font_description (layout, priv->font);
         pango_layout_set_width (layout, PANGO_SCALE * width);
-        pango_layout_set_height (layout, PANGO_SCALE * priv->offset);
         pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
         pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_MIDDLE);
         cairo_move_to (cr, 0, 0);
         gdk_cairo_set_source_color (cr, skin_get_color (aud_active_skin,
          SKIN_PLEDIT_NORMAL));
         pango_cairo_show_layout (cr, layout);
+        g_object_unref(layout);
     }
 
-    PLAYLIST_LOCK (active_playlist);
-    list = active_playlist->entries;
-    list = g_list_nth(list, priv->first);
+    for (i = priv->first; i < priv->first + priv->rows && i < active_length; i ++)
+    {
+        gint i_length = aud_playlist_entry_get_length (active_playlist, i);
 
-    /* It sucks having to run the iteration twice but this is the only
-       way you can reliably get the maximum width so we can get our
-       playlist nice and aligned... -- plasmaroo */
+        if (i_length > 0)
+            g_snprintf (length, sizeof (length), "%d:%02d", i_length / 60000,
+             i_length / 1000 % 60);
+        else
+            length[0] = 0;
 
-    for (i = priv->first;
-         list && i < priv->first + priv->rows;
-         list = g_list_next(list), i++) {
-        entry = list->data;
+        tpadding_dwidth = MAX (tpadding_dwidth, strlen (length));
 
-        if (entry->length != -1)
-        {
-            g_snprintf(length, sizeof(length), "%d:%-2.2d",
-                       entry->length / 60000, (entry->length / 1000) % 60);
-            tpadding_dwidth = MAX(tpadding_dwidth, strlen(length));
-        }
-    }
-
-    /* Reset */
-    list = active_playlist->entries;
-    list = g_list_nth(list, priv->first);
-
-    for (i = priv->first;
-         list && i < priv->first + priv->rows;
-         list = g_list_next(list), i++) {
-        entry = list->data;
-
-        if (entry->selected)
+        if (aud_playlist_entry_get_selected (active_playlist, i))
         {
             gdk_cairo_set_source_color (cr, skin_get_color (aud_active_skin,
              SKIN_PLEDIT_SELECTEDBG));
@@ -470,31 +432,11 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
         }
     }
 
-    list = active_playlist->entries;
-    list = g_list_nth(list, priv->first);
-
-    /* now draw the text */
-    for (i = priv->first;
-         list && i < priv->first + priv->rows;
-         list = g_list_next(list), i++) {
-        entry = list->data;
-
-        /* FIXME: entry->title should NEVER be NULL, and there should
-           NEVER be a need to do a UTF-8 conversion. Playlist title
-           strings should be kept properly. */
-
-        if (!entry->title) {
-            gchar *realfn = g_filename_from_uri(entry->filename, NULL, NULL);
-            gchar *basename = g_path_get_basename(realfn ? realfn : entry->filename);
-            title = aud_filename_to_utf8(basename);
-            g_free(basename); g_free(realfn);
-        }
-        else
-            title = aud_str_to_utf8(entry->title);
-
-        title = aud_convert_title_text(title);
-
-        pos = aud_playlist_get_queue_position (active_playlist, entry);
+    for (i = priv->first; i < priv->first + priv->rows && i < active_length; i ++)
+    {
+        const gchar * title = aud_playlist_entry_get_title (active_playlist, i);
+        gint i_length = aud_playlist_entry_get_length (active_playlist, i);
+        gint pos = aud_playlist_queue_find_entry (active_playlist, i);
 
         tail[0] = 0;
         queuepos[0] = 0;
@@ -503,11 +445,11 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
         if (pos != -1)
             g_snprintf(queuepos, sizeof(queuepos), "%d", pos + 1);
 
-        if (entry->length != -1)
-        {
-            g_snprintf(length, sizeof(length), "%d:%-2.2d",
-                       entry->length / 60000, (entry->length / 1000) % 60);
-        }
+        if (i_length > 0)
+            g_snprintf (length, sizeof (length), "%d:%02d", i_length / 60000,
+             i_length / 1000 % 60);
+        else
+            length[0] = 0;
 
         strncat(tail, length, sizeof(tail) - 1);
         tail_len = strlen(tail);
@@ -524,7 +466,7 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
         else
             tail_width = width;
 
-        if (i == aud_playlist_get_position_nolock (active_playlist))
+        if (i == active_entry)
             gdk_cairo_set_source_color(cr, skin_get_color(aud_active_skin, SKIN_PLEDIT_CURRENT));
         else
             gdk_cairo_set_source_color(cr, skin_get_color(aud_active_skin, SKIN_PLEDIT_NORMAL));
@@ -597,8 +539,6 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
         }
 
         cairo_stroke(cr);
-
-        g_free(title);
     }
 
     if (priv->focused >= priv->first && priv->focused <= priv->first +
@@ -671,31 +611,28 @@ static gboolean ui_skinned_playlist_expose(GtkWidget *widget, GdkEventExpose *ev
         }
     }
 
-    PLAYLIST_UNLOCK (active_playlist);
-
     cairo_destroy(cr);
 
     return FALSE;
 }
 
-static int calc_position (UiSkinnedPlaylistPrivate * private, int y)
+static gint calc_position (UiSkinnedPlaylistPrivate * private, gint y)
 {
-    int position;
+    gint position;
 
-    if (y < 0)
+    if (y < private->offset)
         return -1;
 
     position = private->first + (y - private->offset) / private->row_height;
 
-    if (position > private->first + private->rows - 1 || position >
-     active_length - 1)
+    if (position >= private->first + private->rows || position >= active_length)
         return active_length;
 
     return position;
 }
 
-static int adjust_position (UiSkinnedPlaylistPrivate * private, char relative,
- int position)
+static gint adjust_position (UiSkinnedPlaylistPrivate * private, gboolean
+ relative, gint position)
 {
     if (active_length == 0)
         return -1;
@@ -710,118 +647,58 @@ static int adjust_position (UiSkinnedPlaylistPrivate * private, char relative,
 
     if (position < 0)
         return 0;
-    if (position > active_length - 1)
+    if (position >= active_length)
         return active_length - 1;
 
     return position;
 }
 
-/* This is very inefficient. Don't use it in any loops. */
-static char is_selected (int position)
-{
-    GList * selected;
-    char is;
-
-    selected = aud_playlist_get_selected (active_playlist);
-    is = g_list_find (selected, GINT_TO_POINTER (position)) ? 1 : 0;
-    g_list_free (selected);
-    return is;
-}
-
-static char shift_up_one (void)
-{
-    GList * list;
-
-    PLAYLIST_LOCK (active_playlist);
-    list = active_playlist->entries;
-
-    if (! list || ((PlaylistEntry *) list->data)->selected)
-    {
-        PLAYLIST_UNLOCK (active_playlist);
-        return 0;
-    }
-
-    while (list)
-    {
-        if (((PlaylistEntry *) list->data)->selected)
-            glist_moveup (list);
-
-        list = list->next;
-    }
-
-    PLAYLIST_UNLOCK (active_playlist);
-    return 1;
-}
-
-static char shift_down_one (void)
-{
-    GList * list;
-
-    PLAYLIST_LOCK (active_playlist);
-    list = g_list_last (active_playlist->entries);
-
-    if (! list || ((PlaylistEntry *) list->data)->selected)
-    {
-        PLAYLIST_UNLOCK (active_playlist);
-        return 0;
-    }
-
-    while (list)
-    {
-        if (((PlaylistEntry *) list->data)->selected)
-            glist_movedown (list);
-
-        list = list->prev;
-    }
-
-    PLAYLIST_UNLOCK (active_playlist);
-    return 1;
-}
-
-static void select_single (UiSkinnedPlaylistPrivate * private, char relative,
- int position)
+static void select_single (UiSkinnedPlaylistPrivate * private, gboolean
+ relative, gint position)
 {
     position = adjust_position (private, relative, position);
 
     if (position == -1)
         return;
 
-    aud_playlist_select_all (active_playlist, 0);
-    aud_playlist_select_range (active_playlist, position, position, 1);
+    if (private->focused != -1)
+        aud_playlist_entry_set_selected (active_playlist, private->focused,
+         FALSE);
+
+    if (aud_playlist_selected_count (active_playlist) > 0)
+        aud_playlist_select_all (active_playlist, FALSE);
+
+    aud_playlist_entry_set_selected (active_playlist, position, TRUE);
 
     private->focused = position;
     scroll_to (private, position);
 }
 
-static void select_extend (UiSkinnedPlaylistPrivate * private, char relative,
- int position)
+static void select_extend (UiSkinnedPlaylistPrivate * private, gboolean
+ relative, gint position)
 {
-    int sign;
+    gint count, sign;
 
     position = adjust_position (private, relative, position);
 
     if (position == -1 || position == private->focused)
         return;
 
-    sign = position > private->focused ? 1 : -1;
+    count = adjust_position (private, TRUE, 0);
+    sign = (position > count) ? 1 : -1;
 
-    if (is_selected (private->focused + sign) != is_selected (private->focused))
-        aud_playlist_select_range (active_playlist, private->focused + sign,
-         position, is_selected (private->focused));
-    else if (is_selected (private->focused - sign) != is_selected
-     (private->focused))
-        aud_playlist_select_range (active_playlist, private->focused, position -
-         sign, ! is_selected (private->focused));
-    else
-        aud_playlist_select_range (active_playlist, private->focused, position,
-         1);
+    for (; count != position; count += sign)
+        aud_playlist_entry_set_selected (active_playlist, count,
+         ! aud_playlist_entry_get_selected (active_playlist, count + sign));
+
+    aud_playlist_entry_set_selected (active_playlist, position, TRUE);
 
     private->focused = position;
     scroll_to (private, position);
 }
 
-static void select_slide (UiSkinnedPlaylistPrivate * private, char relative, int
- position)
+static void select_slide (UiSkinnedPlaylistPrivate * private, gboolean relative,
+ gint position)
 {
     position = adjust_position (private, relative, position);
 
@@ -832,119 +709,83 @@ static void select_slide (UiSkinnedPlaylistPrivate * private, char relative, int
     scroll_to (private, position);
 }
 
-static void select_toggle (UiSkinnedPlaylistPrivate * private, char relative,
- int position)
+static void select_toggle (UiSkinnedPlaylistPrivate * private, gboolean
+ relative, gint position)
 {
-    GList * selected;
-
     position = adjust_position (private, relative, position);
 
     if (position == -1)
         return;
 
-    selected = aud_playlist_get_selected (active_playlist);
-    aud_playlist_select_range (active_playlist, position, position, !
-     g_list_find (selected, GINT_TO_POINTER (position)));
-    g_list_free (selected);
+    aud_playlist_entry_set_selected (active_playlist, position,
+     ! aud_playlist_entry_get_selected (active_playlist, position));
 
     private->focused = position;
     scroll_to (private, position);
 }
 
-static void select_move (UiSkinnedPlaylistPrivate * private, char relative, int
- position)
+static void select_move (UiSkinnedPlaylistPrivate * private, gboolean relative,
+ gint position)
 {
     position = adjust_position (private, relative, position);
 
-    if (position == -1)
+    if (private->focused == -1 || position == -1 || position == private->focused)
         return;
 
-    for (; private->focused > position; private->focused --)
-    {
-        if (! shift_up_one ())
-            break;
-    }
-
-    for (; private->focused < position; private->focused ++)
-    {
-        if (! shift_down_one ())
-            break;
-    }
+    private->focused += aud_playlist_shift (active_playlist, private->focused,
+     position - private->focused);
 
     scroll_to (private, private->focused);
 }
 
 static void delete_selected (UiSkinnedPlaylistPrivate * private)
 {
-    GList * selected, * list;
-    int position, shift;
+    gint shift = 0, count;
 
-    position = adjust_position (private, 1, 0);
-
-    if (position == -1)
-        return;
-
-    shift = 0;
-    selected = aud_playlist_get_selected (active_playlist);
-
-    for (list = selected; list; list = list->next)
+    for (count = 0; count < private->focused; count ++)
     {
-        if (GPOINTER_TO_INT (list->data) < position)
+        if (aud_playlist_entry_get_selected (active_playlist, count))
             shift --;
     }
 
-    g_list_free (selected);
+    aud_playlist_delete_selected (active_playlist);
+    active_length = aud_playlist_entry_count (active_playlist);
+    calc_layout (private);
 
-    aud_playlist_delete (active_playlist, 0);
-
-    if (private->first + private->rows > active_length)
-        private->first = active_length - private->rows;
-    if (private->first < 0)
-        private->first = 0;
-
-    if (active_length == 0)
-        private->focused = -1;
-    else
-        select_single (private, 0, position + shift);
+    select_single (private, TRUE, shift);
 }
 
 void ui_skinned_playlist_update (GtkWidget * widget)
 {
-    UiSkinnedPlaylistPrivate * private;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
-
-    if (private->focused > active_length - 1)
-        private->focused = active_length - 1;
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
 
     calc_layout (private);
+
+    if (private->focused != -1)
+        private->focused = adjust_position (private, TRUE, 0);
+
     gtk_widget_queue_draw (widget);
 
-    if (private->slider)
+    if (private->slider != NULL)
         ui_skinned_playlist_slider_update (private->slider);
 }
 
 void ui_skinned_playlist_follow (GtkWidget * widget)
 {
-    UiSkinnedPlaylistPrivate * private;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
 
     cancel_all (widget, private);
-
-    if (active_length == 0)
-        private->focused = -1;
-    else
-        select_single (private, 0, aud_playlist_get_position (active_playlist));
+    select_single (private, FALSE, aud_playlist_get_position (active_playlist));
 
     playlistwin_update ();
 }
 
-char ui_skinned_playlist_key (GtkWidget * widget, GdkEventKey * event)
+gboolean ui_skinned_playlist_key (GtkWidget * widget, GdkEventKey * event)
 {
-    UiSkinnedPlaylistPrivate * private;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
 
     cancel_all (widget, private);
 
@@ -954,162 +795,157 @@ char ui_skinned_playlist_key (GtkWidget * widget, GdkEventKey * event)
         switch (event->keyval)
         {
           case GDK_Up:
-            select_single (private, 1, -1);
+            select_single (private, TRUE, -1);
             break;
           case GDK_Down:
-            select_single (private, 1, 1);
+            select_single (private, TRUE, 1);
             break;
           case GDK_Page_Up:
-            select_single (private, 1, -private->rows);
+            select_single (private, TRUE, - private->rows);
             break;
           case GDK_Page_Down:
-            select_single (private, 1, private->rows);
+            select_single (private, TRUE, private->rows);
             break;
           case GDK_Home:
-            select_single (private, 0, 0);
+            select_single (private, FALSE, 0);
             break;
           case GDK_End:
-            select_single (private, 0, active_length - 1);
+            select_single (private, FALSE, active_length - 1);
             break;
           case GDK_Return:
-            select_single (private, 1, 0);
+            select_single (private, TRUE, 0);
+            aud_playlist_set_playing (active_playlist);
             aud_playlist_set_position (active_playlist, private->focused);
             audacious_drct_play ();
             break;
           case GDK_Escape:
-            select_single (private, 0, aud_playlist_get_position
+            select_single (private, FALSE, aud_playlist_get_position
              (active_playlist));
             break;
           case GDK_Delete:
             delete_selected (private);
             break;
           default:
-            return 0;
+            return FALSE;
         }
         break;
       case GDK_SHIFT_MASK:
         switch (event->keyval)
         {
           case GDK_Up:
-            select_extend (private, 1, -1);
+            select_extend (private, TRUE, -1);
             break;
           case GDK_Down:
-            select_extend (private, 1, 1);
+            select_extend (private, TRUE, 1);
             break;
           case GDK_Page_Up:
-            select_extend (private, 1, -private->rows);
+            select_extend (private, TRUE, -private->rows);
             break;
           case GDK_Page_Down:
-            select_extend (private, 1, private->rows);
+            select_extend (private, TRUE, private->rows);
             break;
           case GDK_Home:
-            select_extend (private, 0, 0);
+            select_extend (private, FALSE, 0);
             break;
           case GDK_End:
-            select_extend (private, 0, active_length - 1);
+            select_extend (private, FALSE, active_length - 1);
             break;
           default:
-            return 0;
+            return FALSE;
         }
         break;
       case GDK_CONTROL_MASK:
         switch (event->keyval)
         {
           case GDK_space:
-            select_toggle (private, 1, 0);
+            select_toggle (private, TRUE, 0);
             break;
           case GDK_Up:
-            select_slide (private, 1, -1);
+            select_slide (private, TRUE, -1);
             break;
           case GDK_Down:
-            select_slide (private, 1, 1);
+            select_slide (private, TRUE, 1);
             break;
           case GDK_Page_Up:
-            select_slide (private, 1, -private->rows);
+            select_slide (private, TRUE, -private->rows);
             break;
           case GDK_Page_Down:
-            select_slide (private, 1, private->rows);
+            select_slide (private, TRUE, private->rows);
             break;
           case GDK_Home:
-            select_slide (private, 0, 0);
+            select_slide (private, FALSE, 0);
             break;
           case GDK_End:
-            select_slide (private, 0, active_length - 1);
+            select_slide (private, FALSE, active_length - 1);
             break;
           default:
-            return 0;
+            return FALSE;
         }
         break;
       case GDK_MOD1_MASK:
         switch (event->keyval)
         {
           case GDK_Up:
-            select_move (private, 1, -1);
+            select_move (private, TRUE, -1);
             break;
           case GDK_Down:
-            select_move (private, 1, 1);
+            select_move (private, TRUE, 1);
             break;
           case GDK_Page_Up:
-            select_move (private, 1, -private->rows);
+            select_move (private, TRUE, -private->rows);
             break;
           case GDK_Page_Down:
-            select_move (private, 1, private->rows);
+            select_move (private, TRUE, private->rows);
             break;
           case GDK_Home:
-            select_move (private, 0, 0);
+            select_move (private, FALSE, 0);
             break;
           case GDK_End:
-            select_move (private, 0, active_length - 1);
+            select_move (private, FALSE, active_length - 1);
             break;
           default:
-            return 0;
+            return FALSE;
         }
         break;
       default:
-        return 0;
+        return FALSE;
     }
 
     playlistwin_update ();
-    return 1;
+    return TRUE;
 }
 
-void ui_skinned_playlist_row_info (GtkWidget * widget, int * rows, int * first)
+void ui_skinned_playlist_row_info (GtkWidget * widget, gint * rows, gint *
+ first, gint * focused)
 {
-    UiSkinnedPlaylistPrivate * private;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
 
     * rows = private->rows;
     * first = private->first;
+    * focused = private->focused;
 }
 
-void ui_skinned_playlist_scroll_to (GtkWidget * widget, int row)
+void ui_skinned_playlist_scroll_to (GtkWidget * widget, gint row)
 {
-    UiSkinnedPlaylistPrivate * private;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
 
     cancel_all (widget, private);
-
     private->first = row;
-
-    if (private->first + private->rows > active_length)
-        private->first = active_length - private->rows;
-    if (private->first < 0)
-        private->first = 0;
+    calc_layout (private);
 
     gtk_widget_queue_draw (widget);
 
-    if (private->slider)
+    if (private->slider != NULL)
         ui_skinned_playlist_slider_update (private->slider);
 }
 
-void ui_skinned_playlist_hover (GtkWidget * widget, int x, int y)
+void ui_skinned_playlist_hover (GtkWidget * widget, gint x, gint y)
 {
-    UiSkinnedPlaylistPrivate * private;
-    int new;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
+    gint new;
 
     if (y < private->offset)
         new = private->first;
@@ -1129,15 +965,14 @@ void ui_skinned_playlist_hover (GtkWidget * widget, int x, int y)
     }
 }
 
-int ui_skinned_playlist_hover_end (GtkWidget * widget)
+gint ui_skinned_playlist_hover_end (GtkWidget * widget)
 {
-    UiSkinnedPlaylistPrivate * private;
-    int temp;
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
+    gint temp = private->hover;
 
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
-
-    temp = private->hover;
     private->hover = -1;
+
     gtk_widget_queue_draw (widget);
     return temp;
 }
@@ -1145,16 +980,14 @@ int ui_skinned_playlist_hover_end (GtkWidget * widget)
 static gboolean ui_skinned_playlist_button_press (GtkWidget * widget,
  GdkEventButton * event)
 {
-    UiSkinnedPlaylistPrivate * private;
-    int position, state;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
+    gint position = calc_position (private, event->y);
+    gint state = event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK |
+     GDK_MOD1_MASK);
 
     cancel_all (widget, private);
 
-    position = calc_position (private, event->y);
-    state = event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK);
-
     switch (event->type)
     {
       case GDK_BUTTON_PRESS:
@@ -1162,68 +995,67 @@ static gboolean ui_skinned_playlist_button_press (GtkWidget * widget,
         {
           case 1:
             if (position == -1 || position == active_length)
-                return 1;
+                return TRUE;
 
             switch (state)
             {
               case 0:
-                if (is_selected (position))
-                    select_slide (private, 0, position);
+                if (aud_playlist_entry_get_selected (active_playlist, position))
+                    select_slide (private, FALSE, position);
                 else
-                    select_single (private, 0, position);
+                    select_single (private, FALSE, position);
 
                 private->drag = DRAG_MOVE;
                 break;
               case GDK_SHIFT_MASK:
-                select_extend (private, 0, position);
+                select_extend (private, FALSE, position);
                 private->drag = DRAG_SELECT;
                 break;
               case GDK_CONTROL_MASK:
-                select_toggle (private, 0, position);
+                select_toggle (private, FALSE, position);
                 private->drag = DRAG_SELECT;
                 break;
               default:
-                return 1;
+                return TRUE;
             }
 
             break;
           case 3:
             if (state)
-                return 1;
+                return TRUE;
 
             if (position != -1 && position != active_length)
             {
-                if (is_selected (position))
-                    select_slide (private, 0, position);
+                if (aud_playlist_entry_get_selected (active_playlist, position))
+                    select_slide (private, FALSE, position);
                 else
-                    select_single (private, 0, position);
+                    select_single (private, FALSE, position);
             }
 
-            ui_manager_popup_menu_show ((GtkMenu *) playlistwin_popup_menu,
-             event->x_root, event->y_root, 3, event->time);
+            ui_popup_menu_show ((position == -1) ? UI_MENU_PLAYLIST :
+             UI_MENU_PLAYLIST_CONTEXT, event->x_root, event->y_root, FALSE,
+             FALSE, 3, event->time);
             break;
           default:
-            return 1;
+            return FALSE;
         }
 
         break;
       case GDK_2BUTTON_PRESS:
         if (event->button != 1 || state || position == -1 || position ==
          active_length)
-            return 1;
+            return TRUE;
 
+        aud_playlist_set_playing (active_playlist);
         aud_playlist_set_position (active_playlist, position);
-
-        if (!audacious_drct_get_playing())
-            audacious_drct_play();
-
+        audacious_drct_play ();
         break;
       default:
-        return 1;
+        return TRUE;
     }
 
     playlistwin_update ();
-    return 1;
+    return TRUE;
 }
 
 static gboolean ui_skinned_playlist_button_release (GtkWidget * widget,
@@ -1231,44 +1063,38 @@ static gboolean ui_skinned_playlist_button_release (GtkWidget * widget,
 {
     cancel_all (widget, UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *)
      widget));
-    return 1;
+    return TRUE;
 }
 
 static gboolean scroll_cb (void * data)
 {
-    UiSkinnedPlaylistPrivate * private;
-    int position;
-
-    private = (UiSkinnedPlaylistPrivate *) data;
-
-    position = adjust_position (private, 1, private->scroll);
+    UiSkinnedPlaylistPrivate * private = data;
+    gint position = adjust_position (private, TRUE, private->scroll);
 
     if (position == -1)
-        return 1;
+        return TRUE;
 
     switch (private->drag)
     {
       case DRAG_SELECT:
-        select_extend (private, 0, position);
+        select_extend (private, FALSE, position);
         break;
       case DRAG_MOVE:
-        select_move (private, 0, position);
+        select_move (private, FALSE, position);
         break;
     }
 
     playlistwin_update ();
-    return 1;
+    return TRUE;
 }
 
 static gboolean ui_skinned_playlist_motion_notify (GtkWidget * widget,
  GdkEventMotion * event)
 {
-    UiSkinnedPlaylistPrivate * private;
-    int position, new_scroll;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
-
-    position = calc_position (private, event->y);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
+    gint position = calc_position (private, event->y);
+    gint new_scroll;
 
     if (private->drag)
     {
@@ -1296,10 +1122,10 @@ static gboolean ui_skinned_playlist_motion_notify (GtkWidget * widget,
             switch (private->drag)
             {
               case DRAG_SELECT:
-                select_extend (private, 0, position);
+                select_extend (private, FALSE, position);
                 break;
               case DRAG_MOVE:
-                select_move (private, 0, position);
+                select_move (private, FALSE, position);
                 break;
             }
 
@@ -1322,29 +1148,27 @@ static gboolean ui_skinned_playlist_motion_notify (GtkWidget * widget,
         }
     }
 
-    return 1;
+    return TRUE;
 }
 
 static gboolean ui_skinned_playlist_leave_notify (GtkWidget * widget,
  GdkEventCrossing * event)
 {
-    UiSkinnedPlaylistPrivate * private;
-
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) widget);
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) widget);
 
     if (! private->drag)
         cancel_all (widget, private);
 
-    return 1;
+    return TRUE;
 }
 
-void ui_skinned_playlist_set_font (GtkWidget * list, char * font)
+void ui_skinned_playlist_set_font (GtkWidget * list, const gchar * font)
 {
-    UiSkinnedPlaylistPrivate * private;
+    UiSkinnedPlaylistPrivate * private = UI_SKINNED_PLAYLIST_GET_PRIVATE
+     ((UiSkinnedPlaylist *) list);
     gchar *font_lower;
 
-    private = UI_SKINNED_PLAYLIST_GET_PRIVATE ((UiSkinnedPlaylist *) list);
-
     private->font = pango_font_description_from_string (font);
 
     text_get_extents(font,
@@ -1359,15 +1183,15 @@ void ui_skinned_playlist_set_font (GtkWidget * list, char * font)
     /* This doesn't take any i18n into account, but i think there is none with TTF fonts
      * FIXME: This can probably be retrieved trough Pango too
      */
-    private->slanted = strstr (font_lower, "oblique") || strstr (font_lower,
-     "italic");
+    private->slanted = strstr (font_lower, "oblique") != NULL || strstr
+     (font_lower, "italic") != NULL;
 
     g_free(font_lower);
 
     calc_layout (private);
     gtk_widget_queue_draw (list);
 
-    if (private->slider)
+    if (private->slider != NULL)
         ui_skinned_playlist_slider_update (private->slider);
 }
 
@@ -1383,18 +1207,21 @@ static gboolean ui_skinned_playlist_popup_show(gpointer data) {
     gint pos = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "popup_position"));
 
     if (GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "timer_active")) == 1 && pos != -1) {
-        Tuple *tuple;
-        Playlist *pl_active = aud_playlist_get_active();
+        const Tuple * tuple;
+        gint pl_active = aud_playlist_get_active ();
         GtkWidget *popup = g_object_get_data(G_OBJECT(widget), "popup");
 
-        tuple = aud_playlist_get_tuple(pl_active, pos);
-        if ((tuple == NULL) || (aud_tuple_get_int(tuple, FIELD_LENGTH, NULL) < 1)) {
-           gchar *title = aud_playlist_get_songtitle(pl_active, pos);
-           audacious_fileinfopopup_show_from_title(popup, title);
-           g_free(title);
-        } else {
-           audacious_fileinfopopup_show_from_tuple(popup , tuple);
+        tuple = aud_playlist_entry_get_tuple (pl_active, pos);
+
+        if (tuple == NULL)
+        {
+            const gchar * title = aud_playlist_entry_get_title (pl_active, pos);
+
+            audacious_fileinfopopup_show_from_title (popup, (gchar *) title);
         }
+        else
+            audacious_fileinfopopup_show_from_tuple (popup, (Tuple *) tuple);
+
         g_object_set_data(G_OBJECT(widget), "popup_active" , GINT_TO_POINTER(1));
     }
 
